/* SwissBetty-TV-Fernbedienung: sound.h */
/* Soundausgabe ber den PWM-Block */

#include "LPC22xx.h"
#include "StdTypes.h"
#include "sound.h"

/*
Die Betty hat keinen dedizierten CODEC eingebaut, weswegen jeder Sound
nur ber den PWM-Block erzeugt werden kann. Daher sind nur 8 Bit Sounds
in Mono mit vorzugsweise 11025 Hz Samplefrequenz mglich.
Andere Sampleraten bis 22050 Hz sind prinzipiell drin, aber unausprobiert.

Zum Einsparen von Speicherplatz kann auch das SOL (Sierra) Format in
der 8 Bit Version benutzt werden.

Jeder Sound mu folgenden Aufbau haben:
- long Anzahl der Samples, bigendian
- long Samplefrequenz, bigendian
- byte Flags, 0=wav, 1=sol
- bytes Samples[]....

 */

SACC SoundTafel; 


long Get_a_Long(char* Pt)
{ long i;
  i = *Pt++;
  i = (i<<8) | *Pt++;
  i = (i<<8) | *Pt++;
  i = (i<<8) | *Pt;
  return i;
}

/* Soundausgabe starten
   ====================

Der Timer PWMTC wird per Prescaler PWMPR aus dem
15 MHz Peripherietakt getaktet.
Er mu von 0 bis mindestens 260 ( 256+2+2 d.h. etwas Slack) zhlen.
Dies ist eine Sample-Zeit.
Fr 22.050 kHz  PWMPR = 2, PWMMR0 = 340
Fr 16.000 kHz  PWMPR = 3, PWMMR0 = 312
Fr 11.025 kHz  PWMPR = 5, PWMMR0 = 272
Fr  8.000 kHz  PWMPR = 7, PWMMR0 = 267

Wir erhhen also den Slack in der Sample-Zeit so weit,
da sich die Samplerate so einigermaen ergibt. Das
macht den Sound etwas leiser.
*/

bool PlaySound ( char* aSound)
{ long L, Frequenz;
  SoundTafel.SampleCount = Get_a_Long(aSound);
  aSound += 4;
  Frequenz = Get_a_Long(aSound);
  aSound += 4;
  SoundTafel.SoundFlags    = *aSound++;
  SoundTafel.SamplePointer = (byte*) aSound;
  SoundTafel.OldSample = 128;
  L = 15000000 / Frequenz;
  L = L / 260;
  if (L<1) return false;

  PWMTCR = 3;                           /* Reset Counter+Prescaler */
  PWMPR = L-1;
  L = 15000000 / (L * Frequenz);

  PWMMR0 = L;	                        /* Samplerate */
  L = (L - 256) / 2;
  SoundTafel.Slack = L;
  PWMMR2 = L +128;	                /* Startwert            */
  PWMLER = 5;                           /* enable MR0 + MR2     */
  PWMMCR = 3;                           /* Interrupt + Counterreset bei MR0 */
  PWMPCR = (1<<10);                     /* enable PWM2-Output   */

  VICIntSelect |= (1<<8);               /* FIQ bei PWM          */
  VICIntEnable  = (1<<8);               /* enable PWM im VIC    */

  FIO0CLR  = (1<<6);                    /* Sound on             */
  PWMTCR   = 9;                         /* enable Counter + PWM */

  FIO0SET  = (1<<11);                   /* NF-Verstrker ein    */

  return true;
}


